"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .accountupdaterinquirysummary import (
    AccountUpdaterInquirySummary,
    AccountUpdaterInquirySummaryTypedDict,
)
from datetime import datetime
from gr4vy.types import BaseModel
from gr4vy.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import List, Literal, Optional
from typing_extensions import Annotated, TypedDict


class AccountUpdaterJobTypedDict(TypedDict):
    id: str
    r"""The ID for the account updater job."""
    merchant_account_id: str
    r"""The ID of the merchant account this job belongs to."""
    inquiries: List[AccountUpdaterInquirySummaryTypedDict]
    r"""A list of the payment methods that have been scheduled for an update."""
    created_at: datetime
    r"""The date and time when this payment method was first created in our system."""
    updated_at: datetime
    r"""The date and time when this payment method was last updated in our system."""
    type: Literal["account-updater-job"]
    r"""Always `account-updater-job`"""


class AccountUpdaterJob(BaseModel):
    id: str
    r"""The ID for the account updater job."""

    merchant_account_id: str
    r"""The ID of the merchant account this job belongs to."""

    inquiries: List[AccountUpdaterInquirySummary]
    r"""A list of the payment methods that have been scheduled for an update."""

    created_at: datetime
    r"""The date and time when this payment method was first created in our system."""

    updated_at: datetime
    r"""The date and time when this payment method was last updated in our system."""

    TYPE: Annotated[
        Annotated[
            Optional[Literal["account-updater-job"]],
            AfterValidator(validate_const("account-updater-job")),
        ],
        pydantic.Field(alias="type"),
    ] = "account-updater-job"
    r"""Always `account-updater-job`"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel
from gr4vy.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal, Optional
from typing_extensions import Annotated, TypedDict


class CryptogramTypedDict(TypedDict):
    cryptogram: str
    r"""The cryptogram of the network token."""
    type: Literal["network-token-cryptogram"]
    r"""Always `network-token-cryptogram`."""


class Cryptogram(BaseModel):
    cryptogram: str
    r"""The cryptogram of the network token."""

    TYPE: Annotated[
        Annotated[
            Optional[Literal["network-token-cryptogram"]],
            AfterValidator(validate_const("network-token-cryptogram")),
        ],
        pydantic.Field(alias="type"),
    ] = "network-token-cryptogram"
    r"""Always `network-token-cryptogram`."""



import uuid
import enum
import base64
import textwrap
import json
from jose import jwk
from cryptography.hazmat.primitives import hashes, serialization
import jwt
from typing import Optional, Dict, Any, Union
from datetime import datetime, timedelta, timezone
from ._version import __user_agent__

class JWTScope(str, enum.Enum):
    READ_ALL = "*.read"
    WRITE_ALL = "*.write"
    EMBED = "embed"
    ANTI_FRAUD_SERVICE_DEFINITIONS_READ = "anti-fraud-service-definitions.read"
    ANTI_FRAUD_SERVICE_DEFINITIONS_WRITE = "anti-fraud-service-definitions.write"
    ANTI_FRAUD_SERVICES_READ = "anti-fraud-services.read"
    ANTI_FRAUD_SERVICES_WRITE = "anti-fraud-services.write"
    API_LOGS_READ = "api-logs.read"
    API_LOGS_WRITE = "api-logs.write"
    APPLE_PAY_CERTIFICATES_READ = "apple-pay-certificates.read"
    APPLE_PAY_CERTIFICATES_WRITE = "apple-pay-certificates.write"
    AUDIT_LOGS_READ = "audit-logs.read"
    AUDIT_LOGS_WRITE = "audit-logs.write"
    BUYERS_READ = "buyers.read"
    BUYERS_WRITE = "buyers.write"
    BUYERS_BILLING_DETAILS_READ = "buyers.billing-details.read"
    BUYERS_BILLING_DETAILS_WRITE = "buyers.billing-details.write"
    CARD_SCHEME_DEFINITIONS_READ = "card-scheme-definitions.read"
    CARD_SCHEME_DEFINITIONS_WRITE = "card-scheme-definitions.write"
    CHECKOUT_SESSIONS_READ = "checkout-sessions.read"
    CHECKOUT_SESSIONS_WRITE = "checkout-sessions.write"
    CONNECTIONS_READ = "connections.read"
    CONNECTIONS_WRITE = "connections.write"
    DIGITAL_WALLETS_READ = "digital-wallets.read"
    DIGITAL_WALLETS_WRITE = "digital-wallets.write"
    FLOWS_READ = "flows.read"
    FLOWS_WRITE = "flows.write"
    GIFT_CARD_SERVICE_DEFINITIONS_READ = "gift-card-service-definitions.read"
    GIFT_CARD_SERVICE_DEFINITIONS_WRITE = "gift-card-service-definitions.write"
    GIFT_CARD_SERVICES_READ = "gift-card-services.read"
    GIFT_CARD_SERVICES_WRITE = "gift-card-services.write"
    GIFT_CARDS_READ = "gift-cards.read"
    GIFT_CARDS_WRITE = "gift-cards.write"
    MERCHANT_ACCOUNTS_READ = "merchant-accounts.read"
    MERCHANT_ACCOUNTS_WRITE = "merchant-accounts.write"
    PAYMENT_LINKS_READ = "payment-links.read"
    PAYMENT_LINKS_WRITE = "payment-links.write"
    PAYMENT_METHOD_DEFINITIONS_READ = "payment-method-definitions.read"
    PAYMENT_METHOD_DEFINITIONS_WRITE = "payment-method-definitions.write"
    PAYMENT_METHODS_READ = "payment-methods.read"
    PAYMENT_METHODS_WRITE = "payment-methods.write"
    PAYMENT_OPTIONS_READ = "payment-options.read"
    PAYMENT_OPTIONS_WRITE = "payment-options.write"
    PAYMENT_SERVICE_DEFINITIONS_READ = "payment-service-definitions.read"
    PAYMENT_SERVICE_DEFINITIONS_WRITE = "payment-service-definitions.write"
    PAYMENT_SERVICES_READ = "payment-services.read"
    PAYMENT_SERVICES_WRITE = "payment-services.write"
    PAYOUTS_READ = "payouts.read"
    PAYOUTS_WRITE = "payouts.write"
    REPORTS_READ = "reports.read"
    REPORTS_WRITE = "reports.write"
    ROLES_READ = "roles.read"
    ROLES_WRITE = "roles.write"
    TRANSACTIONS_READ = "transactions.read"
    TRANSACTIONS_WRITE = "transactions.write"
    USERS_ME_READ = "users.me.read"
    USERS_ME_WRITE = "users.me.write"
    VAULT_FORWARD_READ = "vault-forward.read"
    VAULT_FORWARD_WRITE = "vault-forward.write"
    VAULT_FORWARD_AUTHENTICATIONS_READ = "vault-forward-authentications.read"
    VAULT_FORWARD_AUTHENTICATIONS_WRITE = "vault-forward-authentications.write"
    VAULT_FORWARD_CONFIGS_READ = "vault-forward-configs.read"
    VAULT_FORWARD_CONFIGS_WRITE = "vault-forward-configs.write"
    VAULT_FORWARD_DEFINITIONS_READ = "vault-forward-definitions.read"
    VAULT_FORWARD_DEFINITIONS_WRITE = "vault-forward-definitions.write"
    WEBHOOK_SUBSCRIPTIONS_READ = "webhook-subscriptions.read"
    WEBHOOK_SUBSCRIPTIONS_WRITE = "webhook-subscriptions.write"

JWTScopes = Union[list[JWTScope], list[str]]

def __b64e(value: bytes) -> str:
    return base64.urlsafe_b64encode(value).decode("utf8").strip("=")

def __thumbprint(private_key: str) -> str:
    private_key_pem = textwrap.dedent(private_key).encode()
    serialized_pem = serialization.load_pem_private_key(private_key_pem, password=None)
    key = jwk.construct(serialized_pem, algorithm="ES512").to_dict() # type: ignore
    claims = {k: v for k, v in key.items() if k in {"kty", "crv", "x", "y"}}
    json_claims = json.dumps(claims, separators=(",", ":"), sort_keys=True)
    digest = hashes.Hash(hashes.SHA256())
    digest.update(json_claims.encode("utf8"))
    return str(__b64e(digest.finalize()))

def with_token(private_key: str, scopes: Optional[JWTScopes] = None, expires_in: int = 3600):
    """Generates a new token for every API request.

    Args:
        private_key (str): The RSA private key in string-PEM format.
        scopes (Optional[JWTScopes], optional): List of scopes. If not set, all access will be set as default
        expires_in (int, optional): The expiration time in seconds. Defaults to 3600.
    """
    def callback() -> str:
        return get_token(private_key, scopes, expires_in)
    return callback

def get_token(
    private_key: str,
    scopes: Optional[JWTScopes] = None,
    expires_in: int = 3600,
    embed_params: Optional[Dict[str, Any]] = None,
    checkout_session_id: Optional[str] = None,
) -> str:
    """Generates a token for an API request.

    Args:
        private_key (str): The RSA private key in string-PEM format.
        scopes (Optional[JWTScopes], optional): List of scopes. If not set, all access will be set as default
        expires_in (int, optional): The expiration time in seconds. Defaults to 3600.
        embed_params (Optional[Dict[str, Any]], optional): An optional list of Embed params to pin. Defaults to None.
        checkout_session_id (Optional[str], optional): An optional checkout session ID to link the transaction to. Defaults to None.

    Returns:
        str: A bearer auth token
    """
    if not scopes:
        scopes = [JWTScope.READ_ALL, JWTScope.WRITE_ALL]

    claims: Dict[str, Any] = {
        "scopes": scopes,
        "iss": __user_agent__,
        "iat": datetime.now(tz=timezone.utc),
        "nbf": datetime.now(tz=timezone.utc),
        "exp": datetime.now(tz=timezone.utc) + timedelta(seconds=expires_in),
        "jti": str(uuid.uuid4()),
    }

    if checkout_session_id:
        claims["checkout_session_id"] = checkout_session_id

    if JWTScope.EMBED in scopes and embed_params:
        claims["embed"] = embed_params

    token = jwt.encode(claims, private_key, algorithm="ES512", headers={"kid": __thumbprint(private_key)})
    return token

def update_token(
    token: str,
    private_key: str,
    scopes: Optional[JWTScopes] = None,
    expires_in: int = 3600,
    embed_params: Optional[Dict[str, Any]] = None,
    checkout_session_id: Optional[str] = None,
) -> str:
    """Updates an existing token with a new signature, and optionally new data.

    Args:
        token (str): The previously generated token.
        private_key (str): The RSA private key in string-PEM format.
        scopes (Optional[JWTScopes], optional): List of scopes. If not set, all access will be set as default
        expires_in (int, optional): The expiration time in seconds. Defaults to 3600.
        embed_params (Optional[Dict[str, Any]], optional): An optional list of Embed params to pin. Defaults to None.
        checkout_session_id (Optional[str], optional): An optional checkout session ID to link the transaction to. Defaults to None.

    Returns:
        str: A bearer auth token
    """
    claims: Dict[str, Any] = jwt.decode(token, private_key, algorithms=["ES512"], options={"verify_signature": False})
    previous_scopes: JWTScopes = claims.get("scopes") or []

    return get_token(
        private_key,
        scopes or previous_scopes,
        expires_in,
        embed_params or claims.get("embed"),
        checkout_session_id or claims.get("checkout_session_id"),
    )


def get_embed_token(
    private_key: str,
    embed_params: Optional[Dict[str, Any]] = None,
    checkout_session_id: Optional[str] = None,
) -> str:
    """Generates a token for use with Embed.

    Args:
        private_key (str): The RSA private key in string-PEM format.
        embed_params (Optional[Dict[str, Any]], optional): An optional list of Embed params to pin. Defaults to None.
        checkout_session_id (Optional[str], optional): An optional checkout session ID to link the transaction to. Defaults to None.

    Returns:
        str: A bearer auth token
    """
    return get_token(
        private_key,
        scopes=[JWTScope.EMBED],
        expires_in=3600,
        embed_params=embed_params,
        checkout_session_id=checkout_session_id,
    )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .auditlogaction import AuditLogAction
from .auditlogentries import AuditLogEntries, AuditLogEntriesTypedDict
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import (
    FieldMetadata,
    HeaderMetadata,
    QueryParamMetadata,
    validate_open_enum,
)
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import PlainValidator
from typing import Callable, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ListAuditLogsGlobalsTypedDict(TypedDict):
    merchant_account_id: NotRequired[str]
    r"""The ID of the merchant account to use for this request."""


class ListAuditLogsGlobals(BaseModel):
    merchant_account_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-gr4vy-merchant-account-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The ID of the merchant account to use for this request."""


class ListAuditLogsRequestTypedDict(TypedDict):
    cursor: NotRequired[Nullable[str]]
    r"""A pointer to the page of results to return."""
    limit: NotRequired[int]
    r"""The maximum number of items that are at returned."""
    action: NotRequired[Nullable[AuditLogAction]]
    r"""Filters the results to only the items for which the `audit-log` has an `action` that matches this value."""
    user_id: NotRequired[Nullable[str]]
    r"""Filters the results to only the items for which the `user` has an `id` that matches this value."""
    resource_type: NotRequired[Nullable[str]]
    r"""Filters the results to only the items for which the `audit-log` has a `resource` that matches this type value."""
    merchant_account_id: NotRequired[str]
    r"""The ID of the merchant account to use for this request."""


class ListAuditLogsRequest(BaseModel):
    cursor: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""A pointer to the page of results to return."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 20
    r"""The maximum number of items that are at returned."""

    action: Annotated[
        Annotated[
            OptionalNullable[AuditLogAction], PlainValidator(validate_open_enum(False))
        ],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters the results to only the items for which the `audit-log` has an `action` that matches this value."""

    user_id: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters the results to only the items for which the `user` has an `id` that matches this value."""

    resource_type: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters the results to only the items for which the `audit-log` has a `resource` that matches this type value."""

    merchant_account_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-gr4vy-merchant-account-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The ID of the merchant account to use for this request."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "cursor",
            "limit",
            "action",
            "user_id",
            "resource_type",
            "merchant_account_id",
        ]
        nullable_fields = ["cursor", "action", "user_id", "resource_type"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class ListAuditLogsResponseTypedDict(TypedDict):
    result: AuditLogEntriesTypedDict


class ListAuditLogsResponse(BaseModel):
    next: Callable[[], Optional[ListAuditLogsResponse]]

    result: AuditLogEntries



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.errors import Gr4vyError
from gr4vy.models import errordetail as models_errordetail
from gr4vy.types import BaseModel
from gr4vy.utils import validate_const
import httpx
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import List, Literal, Optional
from typing_extensions import Annotated


class Error405Data(BaseModel):
    TYPE: Annotated[
        Annotated[Optional[Literal["error"]], AfterValidator(validate_const("error"))],
        pydantic.Field(alias="type"),
    ] = "error"
    r"""Always `error`."""

    code: Optional[str] = "method_not_allowed"
    r"""Always `method_not_allowed`"""

    status: Optional[int] = 405
    r"""Always `405`."""

    message: Optional[str] = "Method Not Allowed"
    r"""A human readable message that provides more context to the error."""

    details: Optional[List[models_errordetail.ErrorDetail]] = None
    r"""A list of details that further ellaborate on the error."""


class Error405(Gr4vyError):
    data: Error405Data

    def __init__(
        self,
        data: Error405Data,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        self.data = data



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class WalletPaymentOptionContextTypedDict(TypedDict):
    merchant_name: str
    supported_schemes: List[str]


class WalletPaymentOptionContext(BaseModel):
    merchant_name: str

    supported_schemes: List[str]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .reportcreatortype import ReportCreatorType
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import validate_const, validate_open_enum
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator, PlainValidator
from typing import Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ReportSummaryTypedDict(TypedDict):
    id: str
    r"""The unique ID for the report."""
    merchant_account_id: str
    r"""The merchant account ID this report belongs to."""
    name: str
    r"""The name of the report."""
    type: Literal["report"]
    r"""Always `report`."""
    creator_id: NotRequired[Nullable[str]]
    r"""The ID of the user who created the report."""
    creator_display_name: NotRequired[Nullable[str]]
    r"""The display name of the report creator."""
    creator_type: NotRequired[Nullable[ReportCreatorType]]
    r"""The type of the report creator."""


class ReportSummary(BaseModel):
    id: str
    r"""The unique ID for the report."""

    merchant_account_id: str
    r"""The merchant account ID this report belongs to."""

    name: str
    r"""The name of the report."""

    TYPE: Annotated[
        Annotated[
            Optional[Literal["report"]], AfterValidator(validate_const("report"))
        ],
        pydantic.Field(alias="type"),
    ] = "report"
    r"""Always `report`."""

    creator_id: OptionalNullable[str] = UNSET
    r"""The ID of the user who created the report."""

    creator_display_name: OptionalNullable[str] = UNSET
    r"""The display name of the report creator."""

    creator_type: Annotated[
        OptionalNullable[ReportCreatorType], PlainValidator(validate_open_enum(False))
    ] = UNSET
    r"""The type of the report creator."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["type", "creator_id", "creator_display_name", "creator_type"]
        nullable_fields = ["creator_id", "creator_display_name", "creator_type"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .definitionfieldformat import DefinitionFieldFormat
from gr4vy.types import BaseModel
from gr4vy.utils import validate_open_enum
import pydantic
from pydantic.functional_validators import PlainValidator
from typing_extensions import Annotated, TypedDict


class DefinitionFieldTypedDict(TypedDict):
    r"""A single field that needs to be submitted for a payment service when it is created."""

    key: str
    r"""The key of a field that can be submitted."""
    display_name: str
    r"""The human readable name for this field."""
    required: bool
    r"""Defines if this field is required when the service is created."""
    format_: DefinitionFieldFormat
    secret: bool
    r"""Defines if this field is secret. When `true` the field's value is not returned when querying the payment service information."""


class DefinitionField(BaseModel):
    r"""A single field that needs to be submitted for a payment service when it is created."""

    key: str
    r"""The key of a field that can be submitted."""

    display_name: str
    r"""The human readable name for this field."""

    required: bool
    r"""Defines if this field is required when the service is created."""

    format_: Annotated[
        Annotated[DefinitionFieldFormat, PlainValidator(validate_open_enum(False))],
        pydantic.Field(alias="format"),
    ]

    secret: bool
    r"""Defines if this field is secret. When `true` the field's value is not returned when querying the payment service information."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import UnrecognizedStr
from typing import Literal, Union


GiftCardServiceProvider = Union[
    Literal["mock-gift-card", "qwikcilver-gift-card"], UnrecognizedStr
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from gr4vy import errors, models, utils
from gr4vy._hooks import HookContext
from gr4vy.types import OptionalNullable, UNSET
from gr4vy.utils import get_security_from_env
from gr4vy.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Dict, Mapping, Optional


class Sessions(BaseSDK):
    def google_pay(
        self,
        *,
        origin_domain: str,
        merchant_account_id: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.GooglePaySession:
        r"""Create a Google Pay session

        Create a session for use with Google Pay.

        :param origin_domain: The domain on which Google Pay is being loaded.
        :param merchant_account_id: The ID of the merchant account to use for this request.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateGooglePayDigitalWalletSessionRequest(
            merchant_account_id=merchant_account_id,
            google_pay_session_request=models.GooglePaySessionRequest(
                origin_domain=origin_domain,
            ),
        )

        req = self._build_request(
            method="POST",
            path="/digital-wallets/google/session",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CreateGooglePayDigitalWalletSessionGlobals(
                merchant_account_id=self.sdk_configuration.globals.merchant_account_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.google_pay_session_request,
                False,
                False,
                "json",
                models.GooglePaySessionRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="create_google_pay_digital_wallet_session",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "422",
                "425",
                "429",
                "4XX",
                "500",
                "502",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.GooglePaySession, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.Error400Data, http_res)
            raise errors.Error400(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(errors.Error401Data, http_res)
            raise errors.Error401(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.Error403Data, http_res)
            raise errors.Error403(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.Error404Data, http_res)
            raise errors.Error404(response_data, http_res)
        if utils.match_response(http_res, "405", "application/json"):
            response_data = unmarshal_json_response(errors.Error405Data, http_res)
            raise errors.Error405(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.Error409Data, http_res)
            raise errors.Error409(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "425", "application/json"):
            response_data = unmarshal_json_response(errors.Error425Data, http_res)
            raise errors.Error425(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.Error429Data, http_res)
            raise errors.Error429(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.Error500Data, http_res)
            raise errors.Error500(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(errors.Error502Data, http_res)
            raise errors.Error502(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(errors.Error504Data, http_res)
            raise errors.Error504(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def google_pay_async(
        self,
        *,
        origin_domain: str,
        merchant_account_id: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.GooglePaySession:
        r"""Create a Google Pay session

        Create a session for use with Google Pay.

        :param origin_domain: The domain on which Google Pay is being loaded.
        :param merchant_account_id: The ID of the merchant account to use for this request.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateGooglePayDigitalWalletSessionRequest(
            merchant_account_id=merchant_account_id,
            google_pay_session_request=models.GooglePaySessionRequest(
                origin_domain=origin_domain,
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/digital-wallets/google/session",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CreateGooglePayDigitalWalletSessionGlobals(
                merchant_account_id=self.sdk_configuration.globals.merchant_account_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.google_pay_session_request,
                False,
                False,
                "json",
                models.GooglePaySessionRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="create_google_pay_digital_wallet_session",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "422",
                "425",
                "429",
                "4XX",
                "500",
                "502",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.GooglePaySession, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.Error400Data, http_res)
            raise errors.Error400(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(errors.Error401Data, http_res)
            raise errors.Error401(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.Error403Data, http_res)
            raise errors.Error403(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.Error404Data, http_res)
            raise errors.Error404(response_data, http_res)
        if utils.match_response(http_res, "405", "application/json"):
            response_data = unmarshal_json_response(errors.Error405Data, http_res)
            raise errors.Error405(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.Error409Data, http_res)
            raise errors.Error409(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "425", "application/json"):
            response_data = unmarshal_json_response(errors.Error425Data, http_res)
            raise errors.Error425(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.Error429Data, http_res)
            raise errors.Error429(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.Error500Data, http_res)
            raise errors.Error500(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(errors.Error502Data, http_res)
            raise errors.Error502(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(errors.Error504Data, http_res)
            raise errors.Error504(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def apple_pay(
        self,
        *,
        validation_url: str,
        domain_name: str,
        merchant_account_id: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Dict[str, Any]:
        r"""Create a Apple Pay session

        Create a session for use with Apple Pay.

        :param validation_url: The validation URL as provided by the Apple SDK when processing a payment.
        :param domain_name: The domain on which Apple Pay is being loaded.
        :param merchant_account_id: The ID of the merchant account to use for this request.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateApplePayDigitalWalletSessionRequest(
            merchant_account_id=merchant_account_id,
            apple_pay_session_request=models.ApplePaySessionRequest(
                validation_url=validation_url,
                domain_name=domain_name,
            ),
        )

        req = self._build_request(
            method="POST",
            path="/digital-wallets/apple/session",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CreateApplePayDigitalWalletSessionGlobals(
                merchant_account_id=self.sdk_configuration.globals.merchant_account_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.apple_pay_session_request,
                False,
                False,
                "json",
                models.ApplePaySessionRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="create_apple_pay_digital_wallet_session",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "422",
                "425",
                "429",
                "4XX",
                "500",
                "502",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(Dict[str, Any], http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.Error400Data, http_res)
            raise errors.Error400(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(errors.Error401Data, http_res)
            raise errors.Error401(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.Error403Data, http_res)
            raise errors.Error403(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.Error404Data, http_res)
            raise errors.Error404(response_data, http_res)
        if utils.match_response(http_res, "405", "application/json"):
            response_data = unmarshal_json_response(errors.Error405Data, http_res)
            raise errors.Error405(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.Error409Data, http_res)
            raise errors.Error409(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "425", "application/json"):
            response_data = unmarshal_json_response(errors.Error425Data, http_res)
            raise errors.Error425(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.Error429Data, http_res)
            raise errors.Error429(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.Error500Data, http_res)
            raise errors.Error500(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(errors.Error502Data, http_res)
            raise errors.Error502(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(errors.Error504Data, http_res)
            raise errors.Error504(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def apple_pay_async(
        self,
        *,
        validation_url: str,
        domain_name: str,
        merchant_account_id: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Dict[str, Any]:
        r"""Create a Apple Pay session

        Create a session for use with Apple Pay.

        :param validation_url: The validation URL as provided by the Apple SDK when processing a payment.
        :param domain_name: The domain on which Apple Pay is being loaded.
        :param merchant_account_id: The ID of the merchant account to use for this request.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateApplePayDigitalWalletSessionRequest(
            merchant_account_id=merchant_account_id,
            apple_pay_session_request=models.ApplePaySessionRequest(
                validation_url=validation_url,
                domain_name=domain_name,
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/digital-wallets/apple/session",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CreateApplePayDigitalWalletSessionGlobals(
                merchant_account_id=self.sdk_configuration.globals.merchant_account_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.apple_pay_session_request,
                False,
                False,
                "json",
                models.ApplePaySessionRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="create_apple_pay_digital_wallet_session",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "422",
                "425",
                "429",
                "4XX",
                "500",
                "502",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(Dict[str, Any], http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.Error400Data, http_res)
            raise errors.Error400(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(errors.Error401Data, http_res)
            raise errors.Error401(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.Error403Data, http_res)
            raise errors.Error403(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.Error404Data, http_res)
            raise errors.Error404(response_data, http_res)
        if utils.match_response(http_res, "405", "application/json"):
            response_data = unmarshal_json_response(errors.Error405Data, http_res)
            raise errors.Error405(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.Error409Data, http_res)
            raise errors.Error409(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "425", "application/json"):
            response_data = unmarshal_json_response(errors.Error425Data, http_res)
            raise errors.Error425(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.Error429Data, http_res)
            raise errors.Error429(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.Error500Data, http_res)
            raise errors.Error500(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(errors.Error502Data, http_res)
            raise errors.Error502(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(errors.Error504Data, http_res)
            raise errors.Error504(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def click_to_pay(
        self,
        *,
        checkout_session_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ClickToPaySession:
        r"""Create a Click to Pay session

        Create a session for use with Click to Pay.

        :param checkout_session_id: The checkout session ID to create a Click to Pay session for.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ClickToPaySessionRequest(
            checkout_session_id=checkout_session_id,
        )

        req = self._build_request(
            method="POST",
            path="/digital-wallets/click-to-pay/session",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.ClickToPaySessionRequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="create_click_to_pay_digital_wallet_session",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "422",
                "425",
                "429",
                "4XX",
                "500",
                "502",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.ClickToPaySession, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.Error400Data, http_res)
            raise errors.Error400(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(errors.Error401Data, http_res)
            raise errors.Error401(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.Error403Data, http_res)
            raise errors.Error403(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.Error404Data, http_res)
            raise errors.Error404(response_data, http_res)
        if utils.match_response(http_res, "405", "application/json"):
            response_data = unmarshal_json_response(errors.Error405Data, http_res)
            raise errors.Error405(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.Error409Data, http_res)
            raise errors.Error409(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "425", "application/json"):
            response_data = unmarshal_json_response(errors.Error425Data, http_res)
            raise errors.Error425(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.Error429Data, http_res)
            raise errors.Error429(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.Error500Data, http_res)
            raise errors.Error500(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(errors.Error502Data, http_res)
            raise errors.Error502(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(errors.Error504Data, http_res)
            raise errors.Error504(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def click_to_pay_async(
        self,
        *,
        checkout_session_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ClickToPaySession:
        r"""Create a Click to Pay session

        Create a session for use with Click to Pay.

        :param checkout_session_id: The checkout session ID to create a Click to Pay session for.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ClickToPaySessionRequest(
            checkout_session_id=checkout_session_id,
        )

        req = self._build_request_async(
            method="POST",
            path="/digital-wallets/click-to-pay/session",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.ClickToPaySessionRequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="create_click_to_pay_digital_wallet_session",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "422",
                "425",
                "429",
                "4XX",
                "500",
                "502",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.ClickToPaySession, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.Error400Data, http_res)
            raise errors.Error400(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(errors.Error401Data, http_res)
            raise errors.Error401(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.Error403Data, http_res)
            raise errors.Error403(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.Error404Data, http_res)
            raise errors.Error404(response_data, http_res)
        if utils.match_response(http_res, "405", "application/json"):
            response_data = unmarshal_json_response(errors.Error405Data, http_res)
            raise errors.Error405(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.Error409Data, http_res)
            raise errors.Error409(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "425", "application/json"):
            response_data = unmarshal_json_response(errors.Error425Data, http_res)
            raise errors.Error425(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.Error429Data, http_res)
            raise errors.Error429(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.Error500Data, http_res)
            raise errors.Error500(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(errors.Error502Data, http_res)
            raise errors.Error502(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(errors.Error504Data, http_res)
            raise errors.Error504(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .airline import Airline, AirlineTypedDict
from .applepaypaymentmethodcreate import (
    ApplePayPaymentMethodCreate,
    ApplePayPaymentMethodCreateTypedDict,
)
from .browserinfo import BrowserInfo, BrowserInfoTypedDict
from .cardwithurlpaymentmethodcreate import (
    CardWithURLPaymentMethodCreate,
    CardWithURLPaymentMethodCreateTypedDict,
)
from .cartitem import CartItem, CartItemTypedDict
from .checkoutsessionwithurlpaymentmethodcreate import (
    CheckoutSessionWithURLPaymentMethodCreate,
    CheckoutSessionWithURLPaymentMethodCreateTypedDict,
)
from .clicktopayfpanpaymentmethodcreate import (
    ClickToPayFPANPaymentMethodCreate,
    ClickToPayFPANPaymentMethodCreateTypedDict,
)
from .clicktopaypaymentmethodcreate import (
    ClickToPayPaymentMethodCreate,
    ClickToPayPaymentMethodCreateTypedDict,
)
from .giftcardtokentransactioncreate import (
    GiftCardTokenTransactionCreate,
    GiftCardTokenTransactionCreateTypedDict,
)
from .giftcardtransactioncreate import (
    GiftCardTransactionCreate,
    GiftCardTransactionCreateTypedDict,
)
from .googlepayfpanpaymentmethodcreate import (
    GooglePayFPANPaymentMethodCreate,
    GooglePayFPANPaymentMethodCreateTypedDict,
)
from .googlepaypaymentmethodcreate import (
    GooglePayPaymentMethodCreate,
    GooglePayPaymentMethodCreateTypedDict,
)
from .guestbuyer_input import GuestBuyerInput, GuestBuyerInputTypedDict
from .networktokenpaymentmethodcreate import (
    NetworkTokenPaymentMethodCreate,
    NetworkTokenPaymentMethodCreateTypedDict,
)
from .recipient import Recipient, RecipientTypedDict
from .redirectpaymentmethodcreate import (
    RedirectPaymentMethodCreate,
    RedirectPaymentMethodCreateTypedDict,
)
from .statementdescriptor import StatementDescriptor, StatementDescriptorTypedDict
from .threedsecuredatav1 import ThreeDSecureDataV1, ThreeDSecureDataV1TypedDict
from .threedsecuredatav2 import ThreeDSecureDataV2, ThreeDSecureDataV2TypedDict
from .tokenpaymentmethodcreate import (
    TokenPaymentMethodCreate,
    TokenPaymentMethodCreateTypedDict,
)
from .transactionintent import TransactionIntent
from .transactionpaymentsource import TransactionPaymentSource
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import validate_open_enum
from pydantic import model_serializer
from pydantic.functional_validators import PlainValidator
from typing import Any, Dict, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


TransactionCreatePaymentMethodTypedDict = TypeAliasType(
    "TransactionCreatePaymentMethodTypedDict",
    Union[
        TokenPaymentMethodCreateTypedDict,
        CheckoutSessionWithURLPaymentMethodCreateTypedDict,
        RedirectPaymentMethodCreateTypedDict,
        ClickToPayPaymentMethodCreateTypedDict,
        CardWithURLPaymentMethodCreateTypedDict,
        ApplePayPaymentMethodCreateTypedDict,
        ClickToPayFPANPaymentMethodCreateTypedDict,
        GooglePayFPANPaymentMethodCreateTypedDict,
        GooglePayPaymentMethodCreateTypedDict,
        NetworkTokenPaymentMethodCreateTypedDict,
    ],
)
r"""The optional payment method to use for this transaction. This field is required if no `gift_cards` have been added."""


TransactionCreatePaymentMethod = TypeAliasType(
    "TransactionCreatePaymentMethod",
    Union[
        TokenPaymentMethodCreate,
        CheckoutSessionWithURLPaymentMethodCreate,
        RedirectPaymentMethodCreate,
        ClickToPayPaymentMethodCreate,
        CardWithURLPaymentMethodCreate,
        ApplePayPaymentMethodCreate,
        ClickToPayFPANPaymentMethodCreate,
        GooglePayFPANPaymentMethodCreate,
        GooglePayPaymentMethodCreate,
        NetworkTokenPaymentMethodCreate,
    ],
)
r"""The optional payment method to use for this transaction. This field is required if no `gift_cards` have been added."""


GiftCardUnionTypedDict = TypeAliasType(
    "GiftCardUnionTypedDict",
    Union[GiftCardTokenTransactionCreateTypedDict, GiftCardTransactionCreateTypedDict],
)


GiftCardUnion = TypeAliasType(
    "GiftCardUnion", Union[GiftCardTokenTransactionCreate, GiftCardTransactionCreate]
)


ThreeDSecureDataTypedDict = TypeAliasType(
    "ThreeDSecureDataTypedDict",
    Union[ThreeDSecureDataV2TypedDict, ThreeDSecureDataV1TypedDict],
)
r"""Pass through 3-D Secure data to support external 3-D Secure authorisation. If using an external 3-D Secure provider, you should not pass a `redirect_url` in the `payment_method` object for a transaction."""


ThreeDSecureData = TypeAliasType(
    "ThreeDSecureData", Union[ThreeDSecureDataV2, ThreeDSecureDataV1]
)
r"""Pass through 3-D Secure data to support external 3-D Secure authorisation. If using an external 3-D Secure provider, you should not pass a `redirect_url` in the `payment_method` object for a transaction."""


class TransactionCreateTypedDict(TypedDict):
    amount: int
    r"""The monetary amount for this transaction, in the smallest currency unit for the given currency, for example `1299` cents to create an authorization for `$12.99`. If the `intent` is set to `capture`, an amount greater than zero must be supplied. All gift card amounts are subtracted from this amount before the remainder is charged to the provided `payment_method`."""
    currency: str
    r"""A supported ISO 4217 currency code. For redirect requests, this value must match the one specified for `currency` in `payment_method`."""
    country: NotRequired[Nullable[str]]
    r"""The 2-letter ISO code of the country where the transaction is processed. This is also used to filter the payment services that can process the transaction. If this value is provided for redirect requests and it's not `null`, it must match the one specified for `country` in `payment_method`. Otherwise, the value specified for `country` in `payment_method` will be assumed implicitly."""
    payment_method: NotRequired[Nullable[TransactionCreatePaymentMethodTypedDict]]
    r"""The optional payment method to use for this transaction. This field is required if no `gift_cards` have been added."""
    buyer: NotRequired[Nullable[GuestBuyerInputTypedDict]]
    r"""Guest buyer details provided inline rather than creating a buyer resource beforehand and using the `buyer_id` or `buyer_external_identifier` keys. No buyer resource will be created on Gr4vy when used."""
    buyer_id: NotRequired[Nullable[str]]
    r"""The ID of the buyer to associate this payment method to. If this field is provided then the `buyer_external_identifier` field needs to be unset. If a stored payment method or gift card is provided, then the buyer for that payment method needs to match the buyer for this field."""
    buyer_external_identifier: NotRequired[Nullable[str]]
    r"""The `external_identifier` of the buyer to associate this payment method to. If this field is provided then the `buyer_id` field needs to be unset. If a stored payment method or gift card is provided, then the buyer for that payment method needs to match the buyer for this field."""
    gift_cards: NotRequired[Nullable[List[GiftCardUnionTypedDict]]]
    r"""The optional gift card(s) to use for this transaction. At least one gift card is required if no other `payment_method` has been added. By default, only a maximum limit of 10 gift cards may be used in a single transaction. Please contact our team to change this limit."""
    external_identifier: NotRequired[Nullable[str]]
    r"""An external identifier that can be used to match the transaction against your own records."""
    intent: NotRequired[TransactionIntent]
    store: NotRequired[bool]
    r"""Whether or not to also try and store the payment method with us so that it can be used again for future use. This is only supported for payment methods that support this feature. There are also a few restrictions on how the flag may be set:

    * The flag has to be set to `true` when the `payment_source` is set to `recurring` or `installment`, and `merchant_initiated` is set to `false`.
    * The flag has to be set to `false` (or not set) when using a previously vaulted payment method.
    """
    three_d_secure_data: NotRequired[Nullable[ThreeDSecureDataTypedDict]]
    r"""Pass through 3-D Secure data to support external 3-D Secure authorisation. If using an external 3-D Secure provider, you should not pass a `redirect_url` in the `payment_method` object for a transaction."""
    metadata: NotRequired[Nullable[Dict[str, str]]]
    r"""Any additional information about the transaction that you would like to store as key-value pairs. This data is passed to payment service providers that support it."""
    is_subsequent_payment: NotRequired[bool]
    r"""Indicates whether the transaction represents a subsequent payment coming from a setup recurring payment. Please note there are some restrictions on how this flag may be used.

    The flag can only be `false` (or not set) when the transaction meets one of the following criteria:

    * It is not `merchant_initiated`.
    * `payment_source` is set to `card_on_file`.

    The flag can only be set to `true` when the transaction meets one of the following criteria:
    * It is not `merchant_initiated`.
    * `payment_source` is set to `recurring` or `installment` and `merchant_initiated` is set to `true`.
    * `payment_source` is set to `card_on_file`.
    """
    merchant_initiated: NotRequired[bool]
    r"""Indicates whether the transaction was initiated by the merchant (true) or customer (false)."""
    payment_source: NotRequired[TransactionPaymentSource]
    r"""The way payment method information made it to this transaction."""
    airline: NotRequired[Nullable[AirlineTypedDict]]
    r"""The airline addendum data which describes the airline booking associated with this transaction."""
    cart_items: NotRequired[Nullable[List[CartItemTypedDict]]]
    r"""An array of cart items that represents the line items of a transaction."""
    statement_descriptor: NotRequired[Nullable[StatementDescriptorTypedDict]]
    r"""Details about the payment and the merchant which may end up on the (bank) statement for the payment."""
    previous_scheme_transaction_id: NotRequired[Nullable[str]]
    r"""A scheme's transaction identifier to use in connecting a merchant initiated transaction to a previous customer initiated transaction. If not provided, and a qualifying customer initiated transaction has been previously made with the stored payment method, then Gr4vy will populate this value with the identifier returned for that transaction. This field is also know as the Visa Transaction Identifier, or Mastercard Trace ID."""
    browser_info: NotRequired[Nullable[BrowserInfoTypedDict]]
    r"""Information about the browser used by the buyer. This can be used by anti-fraud services."""
    shipping_details_id: NotRequired[Nullable[str]]
    r"""The unique identifier of a set of shipping details stored for the buyer. If provided, the created transaction will include a copy of the details at the point of transaction creation; i.e. it will not be affected by later changes to the detail in the database."""
    connection_options: NotRequired[Nullable[Dict[str, Dict[str, Any]]]]
    r"""Allows for passing optional configuration per connection to take advantage of connection specific features. When provided, the data is only passed to the target connection type to prevent sharing configuration across connections. Please note that each of the keys this object are in kebab-case, for example `cybersource-anti-fraud` as they represent the ID of the connector. All the other keys will be snake case, for example `merchant_defined_data` or camel case to match an external API that the connector uses."""
    async_capture: NotRequired[bool]
    r"""Whether to capture the transaction asynchronously.

    - When `async_capture` is `false` (default), the transaction is captured in the same request.
    - When `async_capture` is `true`, the transaction is automatically captured at a later time.

    Redirect transactions are not affected by this flag. This flag can only be set to `true` when `intent` is set to `capture`.
    """
    anti_fraud_fingerprint: NotRequired[Nullable[str]]
    r"""This field represents the fingerprint data to be passed to the active anti-fraud service."""
    payment_service_id: NotRequired[Nullable[str]]
    r"""The unique identifier of an existing payment service. When provided, the created transaction will be processed by the given payment service and any routing rules will be skipped."""
    account_funding_transaction: NotRequired[bool]
    r"""Marks the transaction as an AFT. Requires the payment service to support this feature, and might `recipient` and `buyer` data"""
    allow_partial_authorization: NotRequired[bool]
    r"""Defines if the transaction will allow for a partial authorization."""
    recipient: NotRequired[Nullable[RecipientTypedDict]]
    r"""The recipient of any account to account funding. For use with AFTs."""
    installment_count: NotRequired[Nullable[int]]
    r"""The number of installments a buyer is required to make."""


class TransactionCreate(BaseModel):
    amount: int
    r"""The monetary amount for this transaction, in the smallest currency unit for the given currency, for example `1299` cents to create an authorization for `$12.99`. If the `intent` is set to `capture`, an amount greater than zero must be supplied. All gift card amounts are subtracted from this amount before the remainder is charged to the provided `payment_method`."""

    currency: str
    r"""A supported ISO 4217 currency code. For redirect requests, this value must match the one specified for `currency` in `payment_method`."""

    country: OptionalNullable[str] = UNSET
    r"""The 2-letter ISO code of the country where the transaction is processed. This is also used to filter the payment services that can process the transaction. If this value is provided for redirect requests and it's not `null`, it must match the one specified for `country` in `payment_method`. Otherwise, the value specified for `country` in `payment_method` will be assumed implicitly."""

    payment_method: OptionalNullable[TransactionCreatePaymentMethod] = UNSET
    r"""The optional payment method to use for this transaction. This field is required if no `gift_cards` have been added."""

    buyer: OptionalNullable[GuestBuyerInput] = UNSET
    r"""Guest buyer details provided inline rather than creating a buyer resource beforehand and using the `buyer_id` or `buyer_external_identifier` keys. No buyer resource will be created on Gr4vy when used."""

    buyer_id: OptionalNullable[str] = UNSET
    r"""The ID of the buyer to associate this payment method to. If this field is provided then the `buyer_external_identifier` field needs to be unset. If a stored payment method or gift card is provided, then the buyer for that payment method needs to match the buyer for this field."""

    buyer_external_identifier: OptionalNullable[str] = UNSET
    r"""The `external_identifier` of the buyer to associate this payment method to. If this field is provided then the `buyer_id` field needs to be unset. If a stored payment method or gift card is provided, then the buyer for that payment method needs to match the buyer for this field."""

    gift_cards: OptionalNullable[List[GiftCardUnion]] = UNSET
    r"""The optional gift card(s) to use for this transaction. At least one gift card is required if no other `payment_method` has been added. By default, only a maximum limit of 10 gift cards may be used in a single transaction. Please contact our team to change this limit."""

    external_identifier: OptionalNullable[str] = UNSET
    r"""An external identifier that can be used to match the transaction against your own records."""

    intent: Annotated[
        Optional[TransactionIntent], PlainValidator(validate_open_enum(False))
    ] = None

    store: Optional[bool] = False
    r"""Whether or not to also try and store the payment method with us so that it can be used again for future use. This is only supported for payment methods that support this feature. There are also a few restrictions on how the flag may be set:

    * The flag has to be set to `true` when the `payment_source` is set to `recurring` or `installment`, and `merchant_initiated` is set to `false`.
    * The flag has to be set to `false` (or not set) when using a previously vaulted payment method.
    """

    three_d_secure_data: OptionalNullable[ThreeDSecureData] = UNSET
    r"""Pass through 3-D Secure data to support external 3-D Secure authorisation. If using an external 3-D Secure provider, you should not pass a `redirect_url` in the `payment_method` object for a transaction."""

    metadata: OptionalNullable[Dict[str, str]] = UNSET
    r"""Any additional information about the transaction that you would like to store as key-value pairs. This data is passed to payment service providers that support it."""

    is_subsequent_payment: Optional[bool] = False
    r"""Indicates whether the transaction represents a subsequent payment coming from a setup recurring payment. Please note there are some restrictions on how this flag may be used.

    The flag can only be `false` (or not set) when the transaction meets one of the following criteria:

    * It is not `merchant_initiated`.
    * `payment_source` is set to `card_on_file`.

    The flag can only be set to `true` when the transaction meets one of the following criteria:
    * It is not `merchant_initiated`.
    * `payment_source` is set to `recurring` or `installment` and `merchant_initiated` is set to `true`.
    * `payment_source` is set to `card_on_file`.
    """

    merchant_initiated: Optional[bool] = False
    r"""Indicates whether the transaction was initiated by the merchant (true) or customer (false)."""

    payment_source: Annotated[
        Optional[TransactionPaymentSource], PlainValidator(validate_open_enum(False))
    ] = None
    r"""The way payment method information made it to this transaction."""

    airline: OptionalNullable[Airline] = UNSET
    r"""The airline addendum data which describes the airline booking associated with this transaction."""

    cart_items: OptionalNullable[List[CartItem]] = UNSET
    r"""An array of cart items that represents the line items of a transaction."""

    statement_descriptor: OptionalNullable[StatementDescriptor] = UNSET
    r"""Details about the payment and the merchant which may end up on the (bank) statement for the payment."""

    previous_scheme_transaction_id: OptionalNullable[str] = UNSET
    r"""A scheme's transaction identifier to use in connecting a merchant initiated transaction to a previous customer initiated transaction. If not provided, and a qualifying customer initiated transaction has been previously made with the stored payment method, then Gr4vy will populate this value with the identifier returned for that transaction. This field is also know as the Visa Transaction Identifier, or Mastercard Trace ID."""

    browser_info: OptionalNullable[BrowserInfo] = UNSET
    r"""Information about the browser used by the buyer. This can be used by anti-fraud services."""

    shipping_details_id: OptionalNullable[str] = UNSET
    r"""The unique identifier of a set of shipping details stored for the buyer. If provided, the created transaction will include a copy of the details at the point of transaction creation; i.e. it will not be affected by later changes to the detail in the database."""

    connection_options: OptionalNullable[Dict[str, Dict[str, Any]]] = UNSET
    r"""Allows for passing optional configuration per connection to take advantage of connection specific features. When provided, the data is only passed to the target connection type to prevent sharing configuration across connections. Please note that each of the keys this object are in kebab-case, for example `cybersource-anti-fraud` as they represent the ID of the connector. All the other keys will be snake case, for example `merchant_defined_data` or camel case to match an external API that the connector uses."""

    async_capture: Optional[bool] = False
    r"""Whether to capture the transaction asynchronously.

    - When `async_capture` is `false` (default), the transaction is captured in the same request.
    - When `async_capture` is `true`, the transaction is automatically captured at a later time.

    Redirect transactions are not affected by this flag. This flag can only be set to `true` when `intent` is set to `capture`.
    """

    anti_fraud_fingerprint: OptionalNullable[str] = UNSET
    r"""This field represents the fingerprint data to be passed to the active anti-fraud service."""

    payment_service_id: OptionalNullable[str] = UNSET
    r"""The unique identifier of an existing payment service. When provided, the created transaction will be processed by the given payment service and any routing rules will be skipped."""

    account_funding_transaction: Optional[bool] = False
    r"""Marks the transaction as an AFT. Requires the payment service to support this feature, and might `recipient` and `buyer` data"""

    allow_partial_authorization: Optional[bool] = False
    r"""Defines if the transaction will allow for a partial authorization."""

    recipient: OptionalNullable[Recipient] = UNSET
    r"""The recipient of any account to account funding. For use with AFTs."""

    installment_count: OptionalNullable[int] = UNSET
    r"""The number of installments a buyer is required to make."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "country",
            "payment_method",
            "buyer",
            "buyer_id",
            "buyer_external_identifier",
            "gift_cards",
            "external_identifier",
            "intent",
            "store",
            "three_d_secure_data",
            "metadata",
            "is_subsequent_payment",
            "merchant_initiated",
            "payment_source",
            "airline",
            "cart_items",
            "statement_descriptor",
            "previous_scheme_transaction_id",
            "browser_info",
            "shipping_details_id",
            "connection_options",
            "async_capture",
            "anti_fraud_fingerprint",
            "payment_service_id",
            "account_funding_transaction",
            "allow_partial_authorization",
            "recipient",
            "installment_count",
        ]
        nullable_fields = [
            "country",
            "payment_method",
            "buyer",
            "buyer_id",
            "buyer_external_identifier",
            "gift_cards",
            "external_identifier",
            "three_d_secure_data",
            "metadata",
            "airline",
            "cart_items",
            "statement_descriptor",
            "previous_scheme_transaction_id",
            "browser_info",
            "shipping_details_id",
            "connection_options",
            "anti_fraud_fingerprint",
            "payment_service_id",
            "recipient",
            "installment_count",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .method import Method
from .transactionpaymentsource import TransactionPaymentSource
from .transactionstatus import TransactionStatus
from .transactionsummaries import TransactionSummaries, TransactionSummariesTypedDict
from datetime import datetime
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import (
    FieldMetadata,
    HeaderMetadata,
    QueryParamMetadata,
    validate_open_enum,
)
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import PlainValidator
from typing import Callable, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ListTransactionsGlobalsTypedDict(TypedDict):
    merchant_account_id: NotRequired[str]
    r"""The ID of the merchant account to use for this request."""


class ListTransactionsGlobals(BaseModel):
    merchant_account_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-gr4vy-merchant-account-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The ID of the merchant account to use for this request."""


class ListTransactionsRequestTypedDict(TypedDict):
    cursor: NotRequired[Nullable[str]]
    r"""A pointer to the page of results to return."""
    limit: NotRequired[int]
    r"""The maximum number of items that are at returned."""
    created_at_lte: NotRequired[Nullable[datetime]]
    r"""Filters the results to only transactions created before this ISO date-time string. The time zone must be included. Ensure that the date-time string is URL encoded, e.g. `2022-01-01T12:00:00+08:00` must be encoded as `2022-01-01T12%3A00%3A00%2B08%3A00`."""
    created_at_gte: NotRequired[Nullable[datetime]]
    r"""Filters the results to only transactions created after this ISO date-time string. The time zone must be included. Ensure that the date-time string is URL encoded, e.g. `2022-01-01T12:00:00+08:00` must be encoded as `2022-01-01T12%3A00%3A00%2B08%3A00`."""
    updated_at_lte: NotRequired[Nullable[datetime]]
    r"""Filters the results to only transactions updated before this ISO date-time string. The time zone must be included. Ensure that the date-time string is URL encoded, e.g. `2022-01-01T12:00:00+08:00` must be encoded as `2022-01-01T12%3A00%3A00%2B08%3A00`."""
    updated_at_gte: NotRequired[Nullable[datetime]]
    r"""Filters the results to only transactions updated after this ISO date-time string. The time zone must be included. Ensure that the date-time string is URL encoded, e.g. `2022-01-01T12:00:00+08:00` must be encoded as `2022-01-01T12%3A00%3A00%2B08%3A00`."""
    search: NotRequired[Nullable[str]]
    buyer_external_identifier: NotRequired[Nullable[str]]
    buyer_id: NotRequired[Nullable[str]]
    buyer_email_address: NotRequired[Nullable[str]]
    buyer_search: NotRequired[Nullable[List[str]]]
    r"""Filters the results to only get the items for which some of the buyer data contains exactly the provided `buyer_search` values."""
    ip_address: NotRequired[Nullable[str]]
    status: NotRequired[Nullable[List[TransactionStatus]]]
    r"""Filters the results to only the transactions that have a `status` that matches with any of the provided status values."""
    id: NotRequired[Nullable[str]]
    payment_service_transaction_id: NotRequired[Nullable[str]]
    external_identifier: NotRequired[Nullable[str]]
    metadata: NotRequired[Nullable[List[str]]]
    r"""Filters for transactions where their `metadata` values contain all of the provided `metadata` keys. The value sent for `metadata` must be formatted as a JSON string, and all keys and values must be strings. This value should also be URL encoded."""
    amount_eq: NotRequired[Nullable[int]]
    r"""Filters for transactions that have an `amount` that is equal to the provided `amount_eq` value."""
    amount_lte: NotRequired[Nullable[int]]
    r"""Filters for transactions that have an `amount` that is less than or equal to the `amount_lte` value."""
    amount_gte: NotRequired[Nullable[int]]
    r"""Filters for transactions that have an `amount` that is greater than or equal to the `amount_gte` value."""
    currency: NotRequired[Nullable[List[str]]]
    r"""Filters for transactions that have matching `currency` values. The `currency` values provided must be formatted as 3-letter ISO currency code."""
    country: NotRequired[Nullable[List[str]]]
    r"""Filters for transactions that have matching `country` values."""
    payment_service_id: NotRequired[Nullable[List[str]]]
    r"""Filters for transactions that were processed by the provided `payment_service_id` values."""
    payment_method_id: NotRequired[Nullable[str]]
    payment_method_label: NotRequired[Nullable[str]]
    payment_method_scheme: NotRequired[Nullable[List[str]]]
    r"""Filters for transactions where the `payment_method_scheme` matches one of the provided values."""
    payment_method_country: NotRequired[Nullable[str]]
    r"""Filters for transactions that have a payment method with a country that matches with the provided value."""
    payment_method_fingerprint: NotRequired[Nullable[str]]
    method: NotRequired[Nullable[List[Method]]]
    r"""Filters for transactions that have matching `method` values."""
    error_code: NotRequired[Nullable[List[str]]]
    r"""Filters for transactions where the `error_code` matches one for the provided values."""
    has_refunds: NotRequired[Nullable[bool]]
    r"""Filters for transactions with refunds."""
    pending_review: NotRequired[Nullable[bool]]
    r"""Filters for transactions with a pending manual anti-fraud review."""
    checkout_session_id: NotRequired[Nullable[str]]
    r"""Filters for transactions where the `checkout_session_id` matches the provided value."""
    reconciliation_id: NotRequired[Nullable[str]]
    r"""Filters for transactions where the `reconciliation_id` matches the provided value."""
    has_gift_card_redemptions: NotRequired[Nullable[bool]]
    r"""Filters for transactions with gift card redemptions."""
    gift_card_id: NotRequired[Nullable[str]]
    r"""Filters for transactions where a gift card used has an `id` that matches the provided value."""
    gift_card_last4: NotRequired[Nullable[str]]
    r"""Filters for transactions that have at least one gift card redemption where the last 4 digits of its gift card number matches exactly with the provided value."""
    has_settlements: NotRequired[Nullable[bool]]
    r"""Filters for transactions that have at least one associated settlement record."""
    payment_method_bin: NotRequired[Nullable[str]]
    r"""Filter for transactions that have a card with a BIN that matches exactly with the provided value."""
    payment_source: NotRequired[Nullable[List[TransactionPaymentSource]]]
    r"""Filters the results to only the transactions that have a payment source that matches with any of the provided values."""
    is_subsequent_payment: NotRequired[Nullable[bool]]
    r"""Filters for transactions where the `is_subsequent_payment` matches the provided value."""
    merchant_initiated: NotRequired[Nullable[bool]]
    r"""Filters for transactions where the `merchant_initiated` matches the provided value."""
    used_3ds: NotRequired[Nullable[bool]]
    r"""Filters for transactions that attempted 3DS authentication or not."""
    merchant_account_id: NotRequired[str]
    r"""The ID of the merchant account to use for this request."""


class ListTransactionsRequest(BaseModel):
    cursor: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""A pointer to the page of results to return."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 20
    r"""The maximum number of items that are at returned."""

    created_at_lte: Annotated[
        OptionalNullable[datetime],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters the results to only transactions created before this ISO date-time string. The time zone must be included. Ensure that the date-time string is URL encoded, e.g. `2022-01-01T12:00:00+08:00` must be encoded as `2022-01-01T12%3A00%3A00%2B08%3A00`."""

    created_at_gte: Annotated[
        OptionalNullable[datetime],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters the results to only transactions created after this ISO date-time string. The time zone must be included. Ensure that the date-time string is URL encoded, e.g. `2022-01-01T12:00:00+08:00` must be encoded as `2022-01-01T12%3A00%3A00%2B08%3A00`."""

    updated_at_lte: Annotated[
        OptionalNullable[datetime],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters the results to only transactions updated before this ISO date-time string. The time zone must be included. Ensure that the date-time string is URL encoded, e.g. `2022-01-01T12:00:00+08:00` must be encoded as `2022-01-01T12%3A00%3A00%2B08%3A00`."""

    updated_at_gte: Annotated[
        OptionalNullable[datetime],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters the results to only transactions updated after this ISO date-time string. The time zone must be included. Ensure that the date-time string is URL encoded, e.g. `2022-01-01T12:00:00+08:00` must be encoded as `2022-01-01T12%3A00%3A00%2B08%3A00`."""

    search: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET

    buyer_external_identifier: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET

    buyer_id: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET

    buyer_email_address: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET

    buyer_search: Annotated[
        OptionalNullable[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters the results to only get the items for which some of the buyer data contains exactly the provided `buyer_search` values."""

    ip_address: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET

    status: Annotated[
        OptionalNullable[
            List[
                Annotated[TransactionStatus, PlainValidator(validate_open_enum(False))]
            ]
        ],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters the results to only the transactions that have a `status` that matches with any of the provided status values."""

    id: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET

    payment_service_transaction_id: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET

    external_identifier: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET

    metadata: Annotated[
        OptionalNullable[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions where their `metadata` values contain all of the provided `metadata` keys. The value sent for `metadata` must be formatted as a JSON string, and all keys and values must be strings. This value should also be URL encoded."""

    amount_eq: Annotated[
        OptionalNullable[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions that have an `amount` that is equal to the provided `amount_eq` value."""

    amount_lte: Annotated[
        OptionalNullable[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions that have an `amount` that is less than or equal to the `amount_lte` value."""

    amount_gte: Annotated[
        OptionalNullable[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions that have an `amount` that is greater than or equal to the `amount_gte` value."""

    currency: Annotated[
        OptionalNullable[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions that have matching `currency` values. The `currency` values provided must be formatted as 3-letter ISO currency code."""

    country: Annotated[
        OptionalNullable[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions that have matching `country` values."""

    payment_service_id: Annotated[
        OptionalNullable[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions that were processed by the provided `payment_service_id` values."""

    payment_method_id: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET

    payment_method_label: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET

    payment_method_scheme: Annotated[
        OptionalNullable[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions where the `payment_method_scheme` matches one of the provided values."""

    payment_method_country: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions that have a payment method with a country that matches with the provided value."""

    payment_method_fingerprint: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET

    method: Annotated[
        OptionalNullable[
            List[Annotated[Method, PlainValidator(validate_open_enum(False))]]
        ],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions that have matching `method` values."""

    error_code: Annotated[
        OptionalNullable[List[str]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions where the `error_code` matches one for the provided values."""

    has_refunds: Annotated[
        OptionalNullable[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions with refunds."""

    pending_review: Annotated[
        OptionalNullable[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions with a pending manual anti-fraud review."""

    checkout_session_id: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions where the `checkout_session_id` matches the provided value."""

    reconciliation_id: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions where the `reconciliation_id` matches the provided value."""

    has_gift_card_redemptions: Annotated[
        OptionalNullable[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions with gift card redemptions."""

    gift_card_id: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions where a gift card used has an `id` that matches the provided value."""

    gift_card_last4: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions that have at least one gift card redemption where the last 4 digits of its gift card number matches exactly with the provided value."""

    has_settlements: Annotated[
        OptionalNullable[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions that have at least one associated settlement record."""

    payment_method_bin: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter for transactions that have a card with a BIN that matches exactly with the provided value."""

    payment_source: Annotated[
        OptionalNullable[
            List[
                Annotated[
                    TransactionPaymentSource, PlainValidator(validate_open_enum(False))
                ]
            ]
        ],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters the results to only the transactions that have a payment source that matches with any of the provided values."""

    is_subsequent_payment: Annotated[
        OptionalNullable[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions where the `is_subsequent_payment` matches the provided value."""

    merchant_initiated: Annotated[
        OptionalNullable[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions where the `merchant_initiated` matches the provided value."""

    used_3ds: Annotated[
        OptionalNullable[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filters for transactions that attempted 3DS authentication or not."""

    merchant_account_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-gr4vy-merchant-account-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The ID of the merchant account to use for this request."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "cursor",
            "limit",
            "created_at_lte",
            "created_at_gte",
            "updated_at_lte",
            "updated_at_gte",
            "search",
            "buyer_external_identifier",
            "buyer_id",
            "buyer_email_address",
            "buyer_search",
            "ip_address",
            "status",
            "id",
            "payment_service_transaction_id",
            "external_identifier",
            "metadata",
            "amount_eq",
            "amount_lte",
            "amount_gte",
            "currency",
            "country",
            "payment_service_id",
            "payment_method_id",
            "payment_method_label",
            "payment_method_scheme",
            "payment_method_country",
            "payment_method_fingerprint",
            "method",
            "error_code",
            "has_refunds",
            "pending_review",
            "checkout_session_id",
            "reconciliation_id",
            "has_gift_card_redemptions",
            "gift_card_id",
            "gift_card_last4",
            "has_settlements",
            "payment_method_bin",
            "payment_source",
            "is_subsequent_payment",
            "merchant_initiated",
            "used_3ds",
            "merchant_account_id",
        ]
        nullable_fields = [
            "cursor",
            "created_at_lte",
            "created_at_gte",
            "updated_at_lte",
            "updated_at_gte",
            "search",
            "buyer_external_identifier",
            "buyer_id",
            "buyer_email_address",
            "buyer_search",
            "ip_address",
            "status",
            "id",
            "payment_service_transaction_id",
            "external_identifier",
            "metadata",
            "amount_eq",
            "amount_lte",
            "amount_gte",
            "currency",
            "country",
            "payment_service_id",
            "payment_method_id",
            "payment_method_label",
            "payment_method_scheme",
            "payment_method_country",
            "payment_method_fingerprint",
            "method",
            "error_code",
            "has_refunds",
            "pending_review",
            "checkout_session_id",
            "reconciliation_id",
            "has_gift_card_redemptions",
            "gift_card_id",
            "gift_card_last4",
            "has_settlements",
            "payment_method_bin",
            "payment_source",
            "is_subsequent_payment",
            "merchant_initiated",
            "used_3ds",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class ListTransactionsResponseTypedDict(TypedDict):
    result: TransactionSummariesTypedDict


class ListTransactionsResponse(BaseModel):
    next: Callable[[], Optional[ListTransactionsResponse]]

    result: TransactionSummaries



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel
from gr4vy.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal, Optional
from typing_extensions import Annotated, TypedDict


class PaymentMethodStoredCardTypedDict(TypedDict):
    id: str
    r"""The ID of the stored card to use."""
    method: Literal["id"]
    r"""Set to `id` to use a stored card."""


class PaymentMethodStoredCard(BaseModel):
    id: str
    r"""The ID of the stored card to use."""

    METHOD: Annotated[
        Annotated[Optional[Literal["id"]], AfterValidator(validate_const("id"))],
        pydantic.Field(alias="method"),
    ] = "id"
    r"""Set to `id` to use a stored card."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import UnrecognizedStr
from typing import Literal, Union


ReportSpecModel = Union[
    Literal["transactions", "transaction_retries", "detailed_settlement"],
    UnrecognizedStr,
]
